@@ -10,8 +10,7 @@ from pythonwhat.tasks import (
 )
 from protowhat.Test import Test
 from pythonwhat.Test import EqualTest, DefinedCollTest
-from protowhat.Feedback import InstructorError
-from pythonwhat.Feedback import Feedback
+from protowhat.Feedback import Feedback, InstructorError
 from pythonwhat import utils
 from functools import partial
 import re
@@ -53,7 +52,7 @@ def has_part(state, name, msg, fmt_kwargs=None, index=None):
     try:
         verify(state.student_parts[name], index)
     except (KeyError, IndexError):
-        state.report(Feedback(_msg, state))
+        state.report(_msg)
 
     return state
 
@@ -101,7 +100,7 @@ def has_equal_part_len(state, name, unequal_msg):
 
     if d["stu_len"] != d["sol_len"]:
         _msg = state.build_message(unequal_msg, d)
-        state.report(Feedback(_msg, state))
+        state.report(_msg)
 
     return state
 
@@ -195,7 +194,7 @@ def has_equal_ast(state, incorrect_msg=None, code=None, exact=True, append=None)
     if exact and not code:
         state.do_test(EqualTest(stu_rep, sol_rep, Feedback(_msg, state)))
     elif not sol_rep in stu_rep:
-        state.report(Feedback(_msg, state))
+        state.report(_msg)
 
     return state
 
@@ -239,7 +238,7 @@ args_string = """
           {0} of an object after executing the body of e.g. a ``for`` loop.
         copy (bool): whether to try to deep copy objects in the environment, such as lists, that could
           accidentally be mutated. Disable to speed up SCTs. Disabling may lead to cryptic mutation issues.
-        func: custom binary function of form f(stu_result, sol_result), for equality testing.
+        func (function): custom binary function of form f(stu_result, sol_result), for equality testing.
         override: If specified, this avoids the execution of the targeted code in the solution process. Instead, it
           will compare the {0} of the expression in the student process with the value specified in ``override``.
           Typically used in a ``SingleProcessExercise`` or if you want to allow for different solutions other than
@@ -347,13 +346,12 @@ def has_expr(
     if (test == "error") ^ isinstance(eval_stu, Exception):
         fmt_kwargs["stu_str"] = str_stu
         _msg = state.build_message(error_msg, fmt_kwargs, append=append)
-        feedback = Feedback(_msg, state)
-        state.report(feedback)
+        state.report(_msg)
 
     # name is undefined after running expression
     if isinstance(eval_stu, UndefinedValue):
         _msg = state.build_message(undefined_msg, fmt_kwargs, append=append)
-        state.report(Feedback(_msg, state))
+        state.report(_msg)
 
     # test equality of results
     _msg = state.build_message(incorrect_msg, fmt_kwargs, append=append)
@@ -513,8 +511,8 @@ def has_import(
             import matplotlib.pyplot as pltttt
 
     """
-    student_imports = state.ast_dispatcher("imports", state.student_ast)
-    solution_imports = state.ast_dispatcher("imports", state.solution_ast)
+    student_imports = state.ast_dispatcher.find("imports", state.student_ast)
+    solution_imports = state.ast_dispatcher.find("imports", state.solution_ast)
 
     if name not in solution_imports:
         raise InstructorError(
@@ -662,7 +660,7 @@ def has_printout(
         )
 
     try:
-        sol_call_ast = state.ast_dispatcher("function_calls", state.solution_ast)[
+        sol_call_ast = state.ast_dispatcher.find("function_calls", state.solution_ast)[
             "print"
         ][index]["node"]
     except (KeyError, IndexError):
@@ -755,7 +753,7 @@ def has_no_error(
         _msg = state.build_message(
             incorrect_msg, {"error": str(state.reporter.errors[0])}
         )
-        state.report(Feedback(_msg, state))
+        state.report(_msg)
 
     return state
 
